function [ k ] = kern( y, sigma )
% function [ k ] = kern( y, sigma )
%    Returns a gaussian kernel with zero mean and a given sigma (either a
%    covariance matrix, or a variance). Y may be either a vector or a row
%    of vectors.

	d = size(y,1);
	
	if isscalar(sigma)
		sigma = sigma * eye(d); % create the full cov. matrix
	end
	
	ySy = sum( y .* (sigma \ y), 1 );
	k = exp(-ySy/2) / ( (2*pi)^(d/2) * det(sigma)^(1/2) );


end

