function [] = processSequenceIC(seqName, visual)
	if nargin < 2
		visual = false;
	end

	n = 4; % length of the window
	dt = [-4, -7, -10, -13, -16]; % offset to the past
	dtl = length(dt);

	%% prepare the data
	pvaI = load(['/vol/vssp/tragic/track/data/mhtb50_bmc/' seqName '/signalImage.txt']);
	pvaO = load(['/vol/vssp/tragic/track/data/mhtb50_bmc/' seqName '/signalObjectSC.txt']);

	posI = pvaI(:,1:2);
	posO = pvaO(:,1:2);

	len = size(posI, 1);
	x = 1:len;

	posR = posO;
% 	posR = pvaO(randperm(len), 1:2);


	%% plot the input data

if visual
	figure(113)
	subplot 311
	hold off
	plot(x, posI(:,1), 'k');
	hold on
	plot(x, posO(:,1), 'b');
	plot(x, posR(:,1), 'r');
	legend('Image', 'Object', 'Object shuffled', 'Location', 'best');
	xlabel('frame #')
	ylabel('value (px)')
	title('X')
	xrange(0, len);

	subplot 312
	hold off
	plot(x, posI(:,2), 'k');
	hold on
	plot(x, posO(:,2), 'b');
	plot(x, posR(:,2), 'r');
	xlabel('frame #')
	ylabel('value (px)')
	title('Y')
	xrange(0, len);

	% subplot 313
	% hold off
	% plot(x, posI(:,3), 'k');
	% hold on
	% plot(x, posO(:,3), 'b');
	% plot(x, posR(:,3), 'r');
	% xlabel('frame #')
	% ylabel('value (px)')
	% title('S')
	% xrange(0, len);
	drawnow
end


	%% compute the transfer entropy

	I_t{dtl} = [];
	Od_t1 = [];
	Od_t = [];
	Or_t1 = [];
	Or_t = [];
	sigmasd = cell(1,dtl);
	sigmasr = cell(1,dtl);
	i_t{dtl} = [];
	
	TEd = cell(1,dtl);
	TEr = cell(1,dtl);
	for ii = 1:dtl
		TEd{ii} = zeros(1, len);
		TEr{ii} = zeros(1, len);
	end
	
	period = 1;

	d = size(posI, 2);
	maxD = d*n + d*(n+1);
	safeMargin = 10;

	for t = 1 + n + max([dt, 0]) : len + min([dt+1, 0])

		ttt = tic;
		for ii = 1:dtl
			i_t{ii} = flatten(posI(t-n-dt(ii):t-1-dt(ii), :));
			I_t{ii} = [I_t{ii} i_t{ii}];
		end
		od_t1 = flatten(posO(t, :));
		od_t =  flatten(posO(t-n:t-1, :));
% 		or_t1 = flatten(posR(t, :));
% 		or_t =  flatten(posR(t-n:t-1, :));
		tt = randsample(1 + n + max([dt, 0]) : len + min([dt+1, 0]), 1); % t repeated to get at least two-element population
		or_t1 = flatten(posR(tt, :));
		or_t =  flatten(posR(tt-n:tt-1, :));

		Od_t1 = [Od_t1 od_t1];
		Od_t =  [Od_t od_t];
		Or_t1 = [Or_t1 or_t1];
		Or_t =  [Or_t or_t];

		if mod(t, period) == 0
			period = min(max(floor(size(Od_t1, 2)/maxD), 1), 10);
			restart = true;
			if t < 50
				sigmasd = cell(1,dtl);
				sigmasr = cell(1,dtl);
			end
		else
			restart = false;
		end
		if mod(t, 100) == 0
			sigmasd = cell(1,dtl);
			sigmasr = cell(1,dtl);
		end

		if size(Od_t1, 2) > maxD + safeMargin
			for ii = 1:dtl
				[TEd{ii}(t), sigmasd{ii}] = estimTransferEn(Od_t1, Od_t, I_t{ii}, sigmasd{ii}, restart);
				[TEr{ii}(t), sigmasr{ii}] = estimTransferEn(Or_t1, Or_t, I_t{ii}, sigmasr{ii}, restart);
			end
			fprintf('Time t = %d done... (', t);
			for ii = 1:dtl
				if ii > 1
					fprintf(', ');
				end
				fprintf('%.1f/%.1f', TEd{ii}(t), TEr{ii}(t));
			end
			fprintf(') ');
			toc(ttt);
		end
		drawnow
	end


	%% plot the results
if visual
	figure(124)
	hold off
	Hd(1) = plot(TEd{1}, 'b', 'LineWidth', 2);
	hold on
	Hr(1) = plot(TEr{1}, 'b');
	colours = 'bgrmkcy';
	for ii = 2:dtl
		Hd(ii) = plot(TEd{ii}, colours(ii), 'LineWidth', 2);
		Hr(ii) = plot(TEr{ii}, colours(ii));
	end
	xrange(0, len);
	yrange(-5, 10);

	title('TE');
	legends = cell(1,dtl);
	for ii = 1:dtl
		legends{ii} = ['\Delta t = ' num2str(dt(ii))];
	end
	legend(Hd, legends, 'Location', 'NE');
	xlabel('frame #');
	ylabel('entropy (b)');

end



	%% statistical test

	start = find(TEd{1}(1:min(len,100)) == 0, 1, 'last') + 1;

	PP = cell(1,dtl);
	for ii = 1:dtl
		PP{ii} = ones(1, len);
	end

	for i = start+1:len
		start2 = max(start, i-99);
		for ii = 1:dtl
			PP{ii}(i) = ttestWelch(TEd{ii}(start2:i), TEr{ii}(start2:i), true);
		end
	end

	%% plot the permutation test results
if visual
	figure(126);
	hold off
	plot(PP{1}, 'b', 'LineWidth', 2);
	hold on
	for ii = 2:dtl
		plot(PP{ii}, colours(ii), 'LineWidth', 2);
	end
	xrange(0, len);
	yrange(2e-17, 1);
	set(gca, 'yScale', 'log');
end

	%% find the first thing which crosses a threshold

	pvalThr = 1e-4;
	selectedWin = 0;

	for i = 1:len
		for ii = 1:dtl
			if PP{ii}(i) < pvalThr
				selectedWin = ii;
				break
			end
		end
		if PP{ii}(i) < pvalThr
			break
		end
	end

	selectedFrame = i;

	fprintf('The first significant TE was found in window #%i at frame %i.\n', selectedWin, selectedFrame);

	if selectedWin == 0
		fprintf('No causal relationship found!\n');
		dtOptimal = nan;
		nOptimal = nan;
		save(sprintf('resultsIC/mhtb/processed_%s_%s', seqName, datestr(now, 30)),...
		'seqName', 'posI', 'posO', 'TEd', 'TEr', 'PP', 'd', 'len', 'n', 'dt', 'dtl',...
		'selectedWin', 'selectedFrame', 'dtOptimal', 'nOptimal');
		return;
	end

	%% do the grid search

	dts = dt(selectedWin);

	dtm = dts - n;
	dtM = dts + n;

	nm = 1;
	nM = min(2 * n, floor((selectedFrame-d-safeMargin) / (2*d + 1)));

	TEgrid = zeros(dtM-dtm+1, nM-nm+1);

	for dti = dtm:dtM
		for ni = nm:nM
			[I_t, O_t, O_t1] = getPoints(posI', posO', ni, dti, selectedFrame);
			TEgrid(dti-dtm+1, ni-nm+1) = estimTransferEn(O_t1, O_t, I_t);
			fprintf('dt = %d, n = %d done...\n', dti, ni);
		end
		drawnow
	end

	relImpr = relativeImprovement(TEgrid);


	%% plot the grid of TE

	riThr = 0.1;

	% 3D!
if visual
	figure(128);
	hold off
	b3h = bar3(TEgrid, 0.6);
	hold on
	[dti, ni] = find(relImpr > riThr);
	plot3(ni, dti, 0.1 + TEgrid(sub2ind(size(TEgrid), dti, ni)), 'r*')
	xlabel('n')
	ylabel('\Delta t')
	zlabel('TE (b)');
	colorbar
	view([-60, 40]);
	xrange(0.5, size(TEgrid, 2) + 0.5);
	yrange(0.5, size(TEgrid, 1) + 0.5);
	% axis equal
	set(gca, 'XTick', 1:nM-nm+1, 'XTickLabel', nm:nM);
	set(gca, 'YTick', 1:dtM-dtm+1, 'YTickLabel', dtm:dtM);


	for i = 1:length(b3h)
		set(b3h(i), 'CData', kron(relImpr(:,i), ones(6,4)) );
	end
end

	%% get the optimal parameters

	TEf = TEgrid(relImpr > riThr);
	[~, ind] = max(TEf);
	[dti, ni] = find(relImpr > riThr);

	dtOptimal = dti(ind) + dtm - 1;
	nOptimal = ni(ind) + nm - 1;

	fprintf('Optimal parameters are: dt = %d, n = %i.\n', dtOptimal, nOptimal);

	%% save all the outputs
	
	save(sprintf('resultsIC/mhtb/processed_%s_%s', seqName, datestr(now, 30)),...
		'posI', 'posO', 'TEd', 'TEr', 'PP', 'd', 'len', 'n', 'dt', 'dtl',...
		'selectedWin', 'selectedFrame', 'TEgrid', 'relImpr', 'dtOptimal', 'nOptimal');

end




