'''
Script to query a pre-trained gaussian process. Outputting the expected radius
for a set of normalised points on the objects surface.')

command line args:
-i input filepath
-o output filepath
-d num_dimensions
-n num_pts
-v verbose

return codes:
0    success
1    invalid csv file dimensionality
2    File IO failure

Created on Jun 17, 2013

@author: Simon
'''

import argparse
from numpy import *
import GPy
from pickle import Unpickler
import sys

def vprint(flag, text):
    if flag==1:
        for item in text:
            print "verbose: ",item
        print "\n"

if __name__ == '__main__':
    #read command line args
    parser = argparse.ArgumentParser(description='Script to query a pre-trained gaussian process. Outputting the expected radius for a set of normalised points on the objects surface.')
    parser.add_argument('-f',nargs=1,type=str,required=True,help="path to the storage file")
    parser.add_argument('-i',nargs=1,type=str,required=True,help="path to the input CSV file")
    parser.add_argument('-o',nargs=1,type=str,required=True,help="path to the output CSV file, where means and variances will be written")
    parser.add_argument('-d',nargs=1,type=int,default=[-1],help="number of dimensions for input points")
    parser.add_argument('-n',nargs=1,type=int,default=[-1],help="number of input points")
    parser.add_argument('-v',nargs=1,type=int,choices=[0,1],default=[0],help="print out verbose information about training")
    args=parser.parse_args()

    vprint(args.v[0],["Command line arguments were",args])
    if args.v[0]:
        import time

    #read csv file, should be n by d
    vprint(args.v[0],["Loading csv file from ",args.i[0]])
    try:
        csvfile=open(args.i[0])
    except:
        print "Failed to open input CSV file at ",args.i[0]
        sys.exit(2)

    vals=loadtxt(csvfile, float, delimiter=',', ndmin=2)
    vprint(args.v[0],["Loaded matrix had shape",shape(vals)])
    
    if not size(vals,0) == args.n[0] and args.n[0] != -1: # don't check if default
        print "ERROR: Command line arguments specified ",args.n[0],"points but read",size(vals,0),"from file at",args.i
        sys.exit(1)
    if not size(vals,1) == args.d[0] and args.d[0] != -1: # don't check if default
        print "ERROR: Command line arguments specified ",args.d[0],"dimensions but read",size(vals,1),"from file at",args.i,"(should be equal)"
        sys.exit(1)

    #load model
    vprint(args.v[0],["Loading existing model file from", args.f[0]])
    try:
        modelfile=open(args.f[0])
    except:
        print "Failed to open model file at", args.f[0]
        sys.exit(2)
    #load previous model + update bias
    unp=Unpickler(modelfile)
    m=unp.load()
    vprint(args.v[0],["Loaded model had values",m])

    #do query
    vprint(args.v[0],["Performing query"])
    if args.v[0]:
        start=time.time()
    means,uncertainties,upper,lower = m.predict(vals)
    output=transpose(array([means.flatten(),uncertainties.flatten()]))
    if args.v[0]:
        print "prediction time=",time.time()-start,"seconds"

    #write out results
    vprint(args.v[0],["Opening output file at",args.o[0]])
    try:
        outfile=open(args.o[0],"w")
    except:
        print "Failed to open output CSV file at ",args.o[0]
        sys.exit(2)
    savetxt(outfile,output,delimiter=",")
    sys.exit(0)

